#! /usr/bin/perl
#
# wanha - your pesky URL-guardian for IRC
# Copyright (C) 2006-2007 Tommi Saviranta  <wnd@iki.fi>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use warnings FATAL => 'all';
use diagnostics;

use Wanha::HTML;
use Wanha::HTTP;
use Wanha::DB;
use Wanha::Link;
use Wanha::Common;
use Wanha::Lang;


=item main
Main function
=cut
sub main() {
	&Wanha::HTTP::print_header();
	&Wanha::HTML::print_header();

	print "<h1>Sanity check</h1>\n";

	&urlstat();
	&cookies();

	&Wanha::HTML::print_footer(&Wanha::Common::process_time());
}


=item
print URL statistics
=cut
sub urlstat() {
	my ($query, $result);

	# fame = shame + wanha
	# count(url id) = sum(url count) - sum(score fame) - sum(score wanha)

	# url count
	$query = &Wanha::DB::prepare("SELECT COUNT(id) FROM urls");
	$result = &Wanha::DB::query($query);
	my ($total_url_count) = &Wanha::DB::get_row($result);

	# wanha count
	$query = &Wanha::DB::prepare("SELECT COUNT(id), SUM(count) FROM urls " .
			"WHERE count > 1");
	$result = &Wanha::DB::query($query);
	my ($wanha_count, $wanha_sum) = &Wanha::DB::get_row($result);

	# fame, shame, cheat, wanha sum
	$query = &Wanha::DB::prepare("SELECT SUM(fame), SUM(shame), " .
			"SUM(cheat), SUM(wanha) FROM scoreboard");
	$result = &Wanha::DB::query($query);
	my ($total_fame, $total_shame, $total_cheat, $total_wanha) =
			&Wanha::DB::get_row($result);

	# deleted
	$query = &Wanha::DB::prepare(
			"SELECT COUNT(id) FROM urls WHERE chan = -1");
	$result = &Wanha::DB::query($query);
	my ($deleted_urls) = &Wanha::DB::get_row($result);

	my $total_links = $total_url_count + $wanha_sum - $wanha_count;

	# colors
	my $col_wanha_count = &get_color($wanha_count,
				$wanha_sum - $total_fame);
	my $col_total_fame = &get_color($total_fame,
				$total_shame + $total_wanha);
	my $col_cheat = &get_color($total_cheat <= $total_links ? 1 : 0, 1);
	my $col_deleted = &get_color($deleted_urls <= $total_links ? 1 : 0, 1);

	print <<EOF;
<h2>Database</h2>

<table>
	<tr>
		<td>Total URLs</td>
		<td>$total_url_count</td>
	</tr>
	<tr$col_wanha_count>
		<td>Wanha URL count</td>
		<td>$wanha_count</td>
	</tr>
	<tr>
		<td>Wanha URL paste sum</td>
		<td>$wanha_sum</td>
	</tr>
	<tr$col_total_fame>
		<td>Scoreboard fame sum</td>
		<td>$total_fame</td>
	</tr>
	<tr$col_total_fame>
		<td>Scoreboard shame sum</td>
		<td>$total_shame</td>
	</tr>
	<tr$col_total_fame>
		<td>Scoreboard safe wanha sum</td>
		<td>$total_wanha</td>
	</tr>
	<tr$col_cheat>
		<td>Scoreboard cheat sum</td>
		<td>$total_cheat</td>
	</tr>
	<tr$col_deleted>
		<td>Deleted URLs</td>
		<td>$deleted_urls</td>
	</tr>
</table>
EOF
}


=item cookies
Print cookie statistics
=cut
sub cookies() {
	my ($query, $result);
	my $top_n = 10;

	# Cookie count
	#
	$query = &Wanha::DB::prepare("SELECT COUNT(sid) FROM cookies");
	$result = &Wanha::DB::query($query);
	my ($cookie_count) = &Wanha::DB::get_row($result);

	$query = &Wanha::DB::prepare("SELECT COUNT(sid) FROM cookies " .
			"WHERE ip = &sa&");
	$query = &Wanha::DB::set($query, "a", "0.0.0.0");
	$result = &Wanha::DB::query($query);
	my ($cookies_old) = &Wanha::DB::get_row($result);

	$query = &Wanha::DB::prepare("SELECT COUNT(sid) FROM cookies " .
			"WHERE visits > 1");
	$result = &Wanha::DB::query($query);
	my ($cookies_regular) = &Wanha::DB::get_row($result);

	print <<EOF;
<h2>Cookies</h2>

<table>
	<tr>
		<td>Cookies</td>
		<td>$cookie_count</td>
	</tr>
	<tr>
		<td>Old cookies</td>
		<td>$cookies_old</td>
	</tr>
	<tr>
		<td>Regular visitors</td>
		<td>$cookies_regular</td>
	</tr>
</table>
EOF

	# User top-n
	#
	$query = &Wanha::DB::prepare("SELECT ip, set, visits, sid " .
			"FROM cookies " .
			"ORDER BY visits DESC LIMIT &da&");
	$query = &Wanha::DB::set($query, "a", $top_n);
	$result = &Wanha::DB::query($query);

	(&Wanha::DB::row_count($result) == 0) && return;

	print <<EOF;
<h2>Regular visitors</h2>

<table>
	<tr>
		<td>Visits</td>
		<td>Recent IP</td>
		<td>Cookie set</td>
		<td>Session ID</td>
	</tr>
EOF
	while (my ($ip, $set, $visits, $sid) = &Wanha::DB::get_row($result)) {
		print <<EOF;
	<tr>
		<td>$visits</td>
		<td>$ip</td>
		<td>$set</td>
		<td>$sid</td>
	</tr>
EOF
	}
	print "</table>\n";
}


=item get_color
Get color

0: a
1: b
r: row-identifier
=cut
sub get_color($$) {
	my ($a, $b) = @_;
	(defined($a)) || return "";
	if ($a == $b) {
		return " class=\"col_ok\"";
	} elsif ($a < $b) {
		return " class=\"col_le\"";
	} else {
		return " class=\"col_ge\"";
	}
}


&main();
exit(0);
